@@ -124,8 +124,10 @@ class Boto3UploadPostProcessor(UploadPostProcessor):
         if not bucket_exists:
             raise DerivaDownloadError("Target bucket [%s] does not exist." % bucket_name)
 
-        object_qualifier = os.path.basename(self.identity.get("id", "")) or \
-            "anon-" + compute_hashes(self.envars.get("request_ip", "unknown").encode(), hashes=['md5'])['md5'][0]
+        identity = os.path.basename(self.identity.get("id"))
+        object_qualifier = compute_hashes(
+            identity.encode() if identity else
+            "anon-" + self.envars.get("request_ip", "unknown").encode(), hashes=['md5'])['md5'][0]
         if not stob(self.parameters.get("overwrite", "False")):
             object_qualifier = "/".join([object_qualifier, datetime.strftime(datetime.now(), "%Y-%m-%d_%H.%M.%S")])
 
